@@ -19,10 +19,13 @@ import math
 import multiprocessing.pool
 import os
 import os.path as path
+import signal
 import socket
 import subprocess
 import sys
 import tempfile
+import threading
+import time
 import urllib2
 
 import httplib2
@@ -42,7 +45,11 @@ __broadcast_cache = {}  # game eid -> play id -> Play
 __coach_cache = {}  # game eid -> play id -> Play
 
 _xmlf = path.join(path.split(__file__)[0], 'pbp-xml', '%s.xml.gz')
-_xml_base_url = 'http://e2.cdnl3.neulion.com/nfl/edl/nflgr/%d/%s.xml'
+_xml_base_urls = {
+    'default': 'http://e2.cdnl3.neulion.com/nfl/edl/nflgr/%d/%s.xml',
+    '2014': 'http://neulionms-a.akamaihd.net/fs/nfl/nfl/edl/' \
+            'nflgr/%d/%s.xml',
+}
 _coach_url = 'rtmp://neulionms.fcod.llnwd.net/a5306/e1/mp4:' \
              'u/nfl/nfl/coachtapes/%s/%s_all_1600'
 _coach_url = (
@@ -50,8 +57,14 @@ _coach_url = (
     'a5306/e1',
     'mp4:u/nfl/nfl/coachtapes/%s/%s_all_1600',
 )
-_broadcast_url = 'http://nlds82.cdnl3nl.neulion.com/nlds_vod/nfl/vod/' \
-                 '%s/%s/%s/%s/%d_%s_%s_%s_%s_h_%s_%s_%s.mp4.m3u8'
+_broadcast_urls = {
+    'default': 'http://nlds82.cdnl3nl.neulion.com/nlds_vod/nfl/vod/' \
+               '%s/%s/%s/%s/%d_%s_%s_%s_%s_h_%s_%s_%s.mp4.m3u8',
+    '2014': 'http://nlds84.cdnl3nl.neulion.com/nlds_vod/nfl/vod/' \
+               '%s/%s/%s/%s/%d_%s_%s_%s_%s_h_%s_%s_%s.mp4.m3u8',
+    '2011': 'http://nlds53.cdnl3nl.neulion.com/nlds_vod_now/nfl/vod/' \
+               '%s/%s/%s/%s/%d_%s_%s_%s_%s_h_%s_%s_%s.mp4.m3u8',
+}
 
 
 def _eprint(s):
@@ -80,13 +93,14 @@ def broadcast_urls(gobj, quality='1600', condensed=False):
     else:
         stype = 2
 
+    url = _broadcast_urls.get(year, _broadcast_urls['default'])
     kind = 'snap2w' if condensed else 'whole'
     return [
-        _broadcast_url
+        url
         % (year, month, day, gobj.gamekey, stype, gobj.gamekey,
            gobj.away.lower(), gobj.home.lower(), gobj.season(), kind,
            i, quality)
-        for i in ['3', '2', '1a', '1', '4a', '4']
+        for i in ['3', '2', '1a', '1', '4a', '4', '9']
         # We count down here because higher numbers seem to take precedent.
         # For example, the DEN @ NYG game in week 2 of 2013 regular season
         # game. Using `1` links to valid footage that is only ~40 minutes
@@ -101,7 +115,10 @@ def url_status(url):
     URL should be considered valid if and only if its HTTP status is
     `200`.
     """
-    resp, _ = httplib2.Http().request(url, 'HEAD')
+    try:
+        resp, _ = httplib2.Http(timeout=10).request(url, 'HEAD')
+    except socket.timeout:
+        return '404'
     return resp['status']
 
 
@@ -338,14 +355,16 @@ def slice_play(footage_play_dir, full_footage_file, gobj, play,
 
     start_time = '%02d:%02d:%02d.%d' % (st.hh, st.mm, st.ss, st.milli)
     duration = '%02d:%02d:%02d.%d' % (dr.hh, dr.mm, dr.ss, dr.milli)
-    cmd = ['ffmpeg',
-           '-ss', start_time,
-           '-i', full_footage_file,
-           '-acodec', 'copy',
-           '-vcodec', 'copy',
-           '-t', duration,
-           outpath,
-           ]
+    cmd = [
+        'ffmpeg',
+        '-ss', start_time,
+        '-i', full_footage_file,
+        '-acodec', 'copy',
+        '-vcodec', 'copy',
+         '-absf', 'aac_adtstoasc',  # no idea. ffmpeg says I need it though.
+        '-t', duration,
+        outpath,
+    ]
     _run_command(cmd)
 
 
@@ -430,14 +449,15 @@ def download_broadcast(footage_dir, gobj, quality='1600', dry_run=False,
     cmd += ['-i', url]
     if dry_run:
         cmd += ['-t', '30']
-    cmd += ['-absf', 'aac_adtstoasc',  # no idea. ffmpeg says I need it though.
-            '-acodec', 'copy',
-            '-vcodec', 'copy',
-            fp,
-            ]
+    cmd += [
+        '-absf', 'aac_adtstoasc',  # no idea. ffmpeg says I need it though.
+        '-acodec', 'copy',
+        '-vcodec', 'copy',
+        fp,
+    ]
 
     _eprint('Downloading game %s %s' % (gobj.eid, _nice_game(gobj)))
-    if not _run_command(cmd):
+    if not _run_command(cmd, monitor_file=fp):
         _eprint('FAILED to download game %s' % _nice_game(gobj))
     else:
         _eprint('DONE with game %s %s' % (gobj.eid, _nice_game(gobj)))
@@ -506,11 +526,49 @@ def download_coach(footage_dir, gobj, dry_run=False):
                 pass
 
 
-def _run_command(cmd):
+def _file_monitor(pid, f, stop, timeout=60):
+    '''
+    Given a path to a file `f` and a `pid` of a process writing to that
+    file, this will repeatedly check whether a file is increasing in
+    size. If it doesn't increase in size after `timeout` seconds, then
+    `SIGKILL` is sent to `pid`.
+
+    `stop` must be a `threading.Event`. When it is set, this monitor
+    will quit.
+    '''
+    last_size = None
+    while True:
+        if stop.is_set():
+            break
+        try:
+            size = os.stat(f).st_size
+        except OSError:
+            size = 0
+        if size == last_size:
+            _eprint('Download for "%s" has not made progress in %d seconds.\n'
+                    'Killing the download process %d.\n'
+                    '(Tip: Use `nflvid-incomplete` to find incomplete game '
+                    'downloads.)'
+                    % (f, timeout, pid))
+            os.kill(pid, signal.SIGKILL)
+            break
+        last_size = size
+        time.sleep(timeout)
+
+
+def _run_command(cmd, monitor_file=None):
+    monitor, stop = None, None
     try:
         p = subprocess.Popen(cmd,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
+        if monitor_file is not None:
+            stop = threading.Event()
+            monitor = threading.Thread(target=_file_monitor,
+                                       args=(p.pid, monitor_file, stop))
+            monitor.daemon = True  # Don't hold up the program on exit.
+            monitor.start()
+
         output = p.communicate()[0].strip()
 
         if p.returncode != 0:
@@ -529,6 +587,10 @@ def _run_command(cmd):
         _eprint("Could not run '%s' (errno: %d): %s"
                 % (' '.join(cmd), e.errno, e.strerror))
         return False
+    finally:
+        if monitor is not None:
+            stop.set()
+            # No need to wait...
     return output or True
 
 
@@ -834,17 +896,19 @@ def _get_xml_data(eid=None, gamekey=None, fpath=None):
     fpath = _xmlf % eid
     if os.access(fpath, os.R_OK):
         return gzip.open(fpath).read()
+
+    year = int(eid[0:4])
+    month = int(eid[4:6])
+    if month <= 3:
+        year -= 1
+    base = _xml_base_urls.get(str(year), _xml_base_urls['default'])
+    u = base % (year, gamekey)  # The year and the game key.
     try:
-        year = int(eid[0:4])
-        month = int(eid[4:6])
-        if month <= 3:
-            year -= 1
-        u = _xml_base_url % (year, gamekey)  # The year and the game key.
         return urllib2.urlopen(u, timeout=10).read()
     except urllib2.HTTPError, e:
-        _eprint(e)
+        _eprint('%s (%s)' % (e, u))
     except socket.timeout, e:
-        _eprint(e)
+        _eprint('%s (%s)' % (e, u))
     return None
 
 
